@@ -1,9 +1,7 @@
-# Copyright (c) 2017-present, Facebook, Inc.
-# All rights reserved.
-#
-# This source code is licensed under the BSD-style license found in the
-# LICENSE file in the root directory of this source tree. An additional grant
-# of patent rights can be found in the PATENTS file in the same directory.
+# Copyright (c) Facebook, Inc. and its affiliates.
+
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 from __future__ import absolute_import
 from __future__ import division
@@ -107,6 +105,18 @@ class TestDiagnostics(TestCase):
         self.assertAlmostEqual(
             ((df_cv1['yhat'] - df_cv2['yhat']) ** 2).sum(), 0.0)
 
+    def test_cross_validation_uncertainty_disabled(self):
+        df = self.__df.copy()
+        for uncertainty in [0, False]:
+            m = Prophet(uncertainty_samples=uncertainty)
+            m.fit(df, algorithm='Newton')
+            df_cv = diagnostics.cross_validation(
+                m, horizon='4 days', period='4 days', initial='115 days')
+            expected_cols = ['ds', 'yhat', 'y', 'cutoff']
+            self.assertTrue(all(col in expected_cols for col in df_cv.columns.tolist()))
+            df_p = diagnostics.performance_metrics(df_cv)
+            self.assertTrue('coverage' not in df_p.columns)
+
     def test_performance_metrics(self):
         m = Prophet()
         m.fit(self.__df)
@@ -116,7 +126,7 @@ class TestDiagnostics(TestCase):
         df_none = diagnostics.performance_metrics(df_cv, rolling_window=-1)
         self.assertEqual(
             set(df_none.columns),
-            {'horizon', 'coverage', 'mae', 'mape', 'mse', 'rmse'},
+            {'horizon', 'coverage', 'mae', 'mape', 'mdape', 'mse', 'rmse'},
         )
         self.assertEqual(df_none.shape[0], 16)
         # Aggregation level 0
@@ -132,6 +142,7 @@ class TestDiagnostics(TestCase):
         self.assertEqual(df_all.shape[0], 1)
         for metric in ['mse', 'mape', 'mae', 'coverage']:
             self.assertAlmostEqual(df_all[metric].values[0], df_none[metric].mean())
+        self.assertAlmostEqual(df_all['mdape'].values[0], df_none['mdape'].median())
         # Custom list of metrics
         df_horizon = diagnostics.performance_metrics(
             df_cv, metrics=['coverage', 'mse'],
@@ -153,6 +164,11 @@ class TestDiagnostics(TestCase):
             df_cv, metrics=['mape'],
         )
         self.assertIsNone(df_horizon)
+        # List of metrics containing non-valid metrics
+        with self.assertRaises(ValueError):
+            diagnostics.performance_metrics(
+                df_cv, metrics=['mse', 'error_metric'],
+            )
 
     def test_rolling_mean(self):
         x = np.arange(10)
@@ -176,6 +192,29 @@ class TestDiagnostics(TestCase):
         self.assertTrue(np.allclose(np.array([7.]), df['horizon'].values))
         self.assertTrue(np.allclose(np.array([4.5]), df['x'].values))
 
+    def test_rolling_median(self):
+        x = np.arange(10)
+        h = np.arange(10)
+        df = diagnostics.rolling_median_by_h(x=x, h=h, w=1, name='x')
+        self.assertTrue(np.array_equal(x, df['x'].values))
+        self.assertTrue(np.array_equal(h, df['horizon'].values))
+
+        df = diagnostics.rolling_median_by_h(x, h, w=4, name='x')
+        x_true = x[3:] - 1.5
+        self.assertTrue(np.allclose(x_true, df['x'].values))
+        self.assertTrue(np.array_equal(np.arange(3, 10), df['horizon'].values))
+
+        h = np.array([1., 2., 3., 4., 4., 4., 4., 4., 7., 7.])
+        x_true = np.array([1.0, 5.0, 8.0])
+        h_true = np.array([3., 4., 7.])
+        df = diagnostics.rolling_median_by_h(x, h, w=3, name='x')
+        self.assertTrue(np.allclose(x_true, df['x'].values))
+        self.assertTrue(np.array_equal(h_true, df['horizon'].values))
+
+        df = diagnostics.rolling_median_by_h(x, h, w=10, name='x')
+        self.assertTrue(np.allclose(np.array([7.]), df['horizon'].values))
+        self.assertTrue(np.allclose(np.array([4.5]), df['x'].values))
+
     def test_copy(self):
         df = DATA_all.copy()
         df['cap'] = 200.
@@ -227,8 +266,10 @@ class TestDiagnostics(TestCase):
                 self.assertTrue((m1.holidays == m2.holidays).values.all())
             self.assertEqual(m1.country_holidays, m2.country_holidays)
             self.assertEqual(m1.seasonality_mode, m2.seasonality_mode)
-            self.assertEqual(m1.seasonality_prior_scale, m2.seasonality_prior_scale)
-            self.assertEqual(m1.changepoint_prior_scale, m2.changepoint_prior_scale)
+            self.assertEqual(m1.seasonality_prior_scale,
+                             m2.seasonality_prior_scale)
+            self.assertEqual(m1.changepoint_prior_scale,
+                             m2.changepoint_prior_scale)
             self.assertEqual(m1.holidays_prior_scale, m2.holidays_prior_scale)
             self.assertEqual(m1.mcmc_samples, m2.mcmc_samples)
             self.assertEqual(m1.interval_width, m2.interval_width)
